#!/usr/bin/env python
import re,sys
from subprocess import Popen
from sequence_data.io import fasta_chunks
from collections import defaultdict

from optparse import *

usage = """
usage: %prog [options] mature.fa targets.fa [families.txt]

Reads miRNA target sequences and mature miRNA sequences (optionally uses familiy names to fix the assignment). 
Then aligns each family member to the target site with RNAduplex. 
"""

parser = OptionParser(usage=usage)
parser.add_option("-S","--system",dest="system",default="hg19",help="model system (default=hg19)")
parser.add_option("-f","--families",dest="families",default='mirnas_min3vertebrates.hsa_mirs.families',help="mirna family assignment (1 row=1 family, comma-separated members)")
parser.add_option("-m","--mature",dest="mature",default='human_mature.fa',help="mature miRNA sequences (fasta)")
parser.add_option("-t","--targets",dest="targets",default='pictar2_mammals_hg19.fna',help="target sequences (fasta)")
options,args = parser.parse_args()

from sequence_data import systems
system = getattr(systems,options.system)

def read_families(fname=options.families):
    family_members = defaultdict(set)
    for line in file(fname):
        mirs = set([mir.replace('hsa-','').replace('mmu-','') for mir in line.rstrip().split(',') if mir.strip()])
        #print line,mirs
        for m in mirs:
            family_members[m] |= mirs

    return family_members
                

def read_mirnas(fname=options.mature):
    mature = {}
    for fa_id,seq in fasta_chunks(file(fname)):
        # last part of fa id is short miR-name
        mir = fa_id.rstrip().split()[-1]
        #print "'%s'" % mir
        mature[mir] = seq.upper().replace('T','U')

    return mature

def assign_energies(fname=options.targets):
    print "# loading"
    mature = read_mirnas()
    family_members = read_families()
    print "# done"
    from sequence_data.lowlevel import rnalib
    RD = rnalib.RNAduplex()
    
    for fa_id,target in fasta_chunks(file(fname)):
        family,score,loc = fa_id.rstrip().split()
        target = target.upper().replace('T','U')
        
        chrom,strand,start,end = re.match(r'^loc=(?P<chrom>chr\w+)(?P<sense>[\+,\-])\:(?P<start>\d+)\-(?P<end>\d+)$',loc).groups()
        
        #print "target sequence for '%s' is" % fa_id
        #print target
        
        head = family.split('/')[0]
        #print head, family_members[head]
        for mir_name in sorted(family_members[head]):
            #print "hybridizing '%s'" % mir_name
            mir = mature[mir_name]
            #print mir
            E,i,j,struct = RD.fold(target,mir)
            
            bp = struct.count('(')
            out = [chrom,mir,"pictar2_cons_mammals_hg19",start,end,str(E),strand,str(bp),'name="%s__%s"; target="%s"; mir="%s"; struct="%s";' % (family,mir_name,family,mir_name,struct)]
            print "\t".join(out)
            

assign_energies()